<?php
	// ---------------------------------------------------------------------------
	// this function attempts to set up a connection to the supplied ftp server
	// using the given login information, port number, and passive mode flag. An
	// ftp handle is returned on success. Otherwise, an array of errors is returned.
	function get_ftp_connection($ftp_server_name, $ftp_username, $ftp_password, $ftp_port_num = 21, $ftp_passive_mode = 0){
		$error_log = Array();
		$ftp_handle = null;
		$is_logged_in = FALSE;

		// validate the port number passed in
		if(!(is_numeric($ftp_port_num) && $ftp_port_num > 1 && $ftp_port_num < 65535)){ $ftp_port_num = 21; }

		// make a correct bool value for the passive mode param
		$ftp_passive_mode = ($ftp_passive_mode) ? TRUE : FALSE;

		// make sure PHP has been installed with FTP functionality
		if(function_exists('ftp_connect') == TRUE){
			// attempt to connect to the ftp server
			$ftp_handle = @ftp_connect($ftp_server_name, $ftp_port_num);
			if($ftp_handle !== FALSE){
				// attempt to login to the ftp server with the supplied username and password
				$is_logged_in = @ftp_login($ftp_handle, $ftp_username, $ftp_password);
				if($is_logged_in){
					// set the passive mode accordingly
					if($ftp_passive_mode){
						if(!@ftp_pasv($ftp_handle, $ftp_passive_mode)){
							$error_log[] = "Could not switch passive mode for '$ftp_server_name'.";
						}
					} // end if
				} // end if
				else{ $error_log[] = "Could not log into '$ftp_server_name'. Make sure the username and password supplied are correct."; }
			} // end if
			else{ $error_log[] = "Could not connect to '$ftp_server_name' on port # ${ftp_port_num}."; }
		} // end if
		else{ $error_log[] = "PHP's FTP module has not been installed. Please contact your System Administrator."; }

		// if errors were found, return the error array. otherwise, return the handle to our ftp connection
		return (count($error_log) > 0) ? $error_log : $ftp_handle;
	} // end function
	// ---------------------------------------------------------------------------
	// given an existing ftp connection, this function attempts to change to a
	// directory and read the contents of that directory. An associative array
	// is returned where the key 'ftp_directory' points to the actually
	// directory that was searched (may be different than supplied). The keys
	// 'ftp_dirs', 'ftp_files', and 'ftp_symlinks' point to arrays of datastructures that contain information
	// about each entry in the directory listing (size, group, etc.)
	function get_ftp_list($ftp_handle, $ftp_directory){
		// arrays to store the directory contents
		$ftp_dir_array = Array();
		$ftp_file_array = Array();
		$ftp_symlink_array = Array();

		// the master datastructure to be returned by the function
		$ftp_list_results = Array(
			'ftp_directory' => '',
			'ftp_dirs' 		=> Array(),
			'ftp_files' 	=> Array(),
			'ftp_symlinks' 	=> Array()
		);

		// try to chdir to the root directory
		if($ftp_directory == '/'){
			@ftp_chdir($ftp_handle, $ftp_directory);
		}
		// an empty directory might mean the user is directed to their home
		// directory, so we need to find out what it is.
		elseif($ftp_directory == ''){
			@ftp_chdir($ftp_handle, $ftp_directory);
			$ftp_directory = @ftp_pwd($ftp_handle);
		}
		// otherwise it's a normal directory listing
		else{
			$ftp_directory = str_replace("\'", "\\\'", $ftp_directory);

			// try to change to the directory the first time
			if(FALSE == @ftp_chdir($ftp_handle, $ftp_directory)){
				// chdir failed, so trim the slashes from the dir
				$ftp_directory = trim_dir_slashes($ftp_directory);
				// try to connect again
				if(FALSE == @ftp_chdir($ftp_handle, $ftp_directory)){
					// everything else has failed, so go to the root dir
					$ftp_directory = "/";
					@ftp_chdir($ftp_handle, $ftp_directory);
				} // end if
			} // end if
		} // end else

		// remember the actual directory searched
		$ftp_list_results['ftp_directory'] = $ftp_directory;

		// first, try to get all of the files and directories including hidden
		// files (-a flag used)
		$rawlist_array = ftp_rawlist($ftp_handle, '-a');

		// some ftp servers don't return anything with the -a flag, so try again
		if(count($rawlist_array) <= 1){
			$rawlist_array = ftp_rawlist($ftp_handle, '');
		}

		// parse all of the rawlist entries found and place the data into the
		// appropriate array
		foreach($rawlist_array as $idx => $rawlist_str){
			$rawlist_struct = parse_ftp_rawlist_line($rawlist_str);
			// if the struct contains a valid entry
			if($rawlist_struct['name'] != '' && $rawlist_struct['type'] != ''){
				// sort the rawlist entry based on it's type
				switch($rawlist_struct['type']){
					case 'd': $ftp_list_results['ftp_dirs'][] = $rawlist_struct; break;
					case '-': $ftp_list_results['ftp_files'][] = $rawlist_struct; break;
					case 'l': $ftp_list_results['ftp_symlinks'][] = $rawlist_struct; break;
				} // end switch
			} // end if
		} // end foreach

		return $ftp_list_results;
	} // end function
	// ---------------------------------------------------------------------------
	// this funciton takes a directory name as a string and attempts to remove
	// any leading or trailing forward or backslashes.
	function trim_dir_slashes($ftp_directory){
		$ftp_directory = trim($ftp_directory);
		// remove leading slashes
		$char = substr($ftp_directory,0,1);
		if($char == "/" || $char == "\\"){ $ftp_directory = substr($ftp_directory,1,strlen($ftp_directory)-1); }
		// remove trailing slashes
		$char = substr($ftp_directory,strlen($ftp_directory)-1,1);
		if($char == "/" || $char == "\\"){ $ftp_directory = substr($ftp_directory,0,strlen($ftp_directory)-1); }
		return $ftp_directory;
	}
	// ---------------------------------------------------------------------------
	// since there is no standardized way to display the contents of an FTP
	// LIST command, things get hairy. We have to attempt to parse the raw
	// output with a series of regular expressions. These regex's have been
	// compiled from a number of sources online including donated regexs from
	// forum conversations. PHP.net also has a lot of tips
	// about this subject as many other people have run into the problem before.
	// This function returns an assoc. array of fields extracted from the raw output
	function parse_ftp_rawlist_line($rawlist_str){
		// flag if we've successfully matched the rawlist string
		$match_found = 0;

		// set up our data struct to be populated and returned.
		$rawlist_struct = Array("type" => "", "perm" => "", "group" => "", "owner" => "", "size" => "", "time" => "", "name" => "");

		// define our array of regular expressions to test the raw output line
		$regex_array = Array(
			"([-dl])([rwxst-]{9})[ ]+([0-9]+)[ ]+([^ ]+)[ ]+(.+)[ ]+([0-9]+)[ ]+([a-zA-Z]+[ ]+[0-9]+)[ ]+([0-9:]+)[ ]+(.*)", // strict match
			"([-dl])([rwxst-]{9})[ ]+(.*)[ ]+([a-zA-Z0-9 ]+)[ ]+([0-9:]+)[ ]+(.*)", // less strict match
			"([0-9/-]+)[ ]+([0-9:AMP]+)[ ]+([0-9]*)[ ]+(.*)", // windows
			"([-]|[d])[ ]+(.{10})[ ]+([^ ]+)[ ]+([0-9]*)[ ]+([a-zA-Z]*[ ]+[0-9]*)[ ]+([0-9:]*)[ ]+(.*)", // Netware
			"([-dl])([rwxst-]{9})[ ]+([0-9]+)[ ]+([a-zA-Z0-9]+)[ ]+([a-zA-Z0-9]+)[ ]+([0-9]+)[ ]+([a-zA-Z]+[ ]+[0-9]+)[ ]+([0-9:]+)[ ](.*)", // Titan
			"([a-zA-Z0-9_-]+)[ ]+([0-9]+)[ ]+([0-9/-]+)[ ]+([0-9:]+)[ ]+([a-zA-Z0-9_ -\*]+)[ /]+([^/]+)" // as400
		);

		// make sure a string was passed in
		if(strlen($rawlist_str) > 0){
			// loop through each regex until we find a match (if any)
			foreach($regex_array as $match_idx => $regex){
				if(ereg($regex, $rawlist_str, $matches) == TRUE){
					$match_found = 1;
					break;
				}
			} // end foreach
			// if a match was found, attempt to parse out the data
			if($match_found){
				switch($match_idx){
					case 0: // strict match
						$rawlist_struct['type'] = $matches[1];
						$rawlist_struct['perm'] = $matches[2];
						$rawlist_struct['owner'] = $matches[4];
						$rawlist_struct['group'] = $matches[5];
						$rawlist_struct['size'] = $matches[6];
						$rawlist_struct['time'] = "$matches[7] $matches[8]";
						$rawlist_struct['name'] = $matches[9];
					break;
					case 1: // less strict match
						$rawlist_struct['type'] = $matches[1];
						$rawlist_struct['perm'] = $matches[2];
						$rawlist_struct['size'] = $matches[3];
						$rawlist_struct['time'] = "$matches[4] $matches[5]";
						$rawlist_struct['name'] = $matches[6];
					break;
					case 2: // windows
						$rawlist_struct['time'] = "$matches[1] $matches[2]";
						$rawlist_struct['size'] = $matches[3];
						$rawlist_struct['name'] = $matches[4];
						// if a size was found, then it must be a file, otherwise a directory
						$rawlist_struct['type'] = ($rawlist_struct['size'] != '') ? '-' : 'd';
						// windows systems can have "<DIR>" prepended to the name, so remove it
						if($rawlist_struct['type'] == 'd'){
							$rawlist_struct['name'] = trim(str_replace("<DIR>","",str_replace("<dir>","",$rawlist_struct['name'])));
						}
					break;
					case 3: // netware 
						$rawlist_struct['type'] = $matches[1];
						$rawlist_struct['perm'] = $matches[2];
						$rawlist_struct['owner'] = $matches[3];
						$rawlist_struct['size'] = $matches[4];
						$rawlist_struct['time'] = "$matches[5] $matches[6]";
						$rawlist_struct['name'] = $matches[7];
					break;
					case 4: // as400 
						$rawlist_struct['owner'] = $matches[1];
						$rawlist_struct['size'] = $matches[2];
						$rawlist_struct['time'] = "$matches[3] $matches[4]";
						$rawlist_struct['type'] = ($matches[5] == "*STMF") ? "-" : "d";
						$rawlist_struct['name'] = $matches[6];
					break;
					case 5: // titan
						$rawlist_struct['type'] = $matches[1];
						$rawlist_struct['perm'] = $matches[2];
						$rawlist_struct['owner'] = $matches[4];
						$rawlist_struct['group'] = $matches[5];
						$rawlist_struct['size'] = $matches[6];
						$rawlist_struct['time'] = "$matches[7] $matches[8]";
						$rawlist_struct['name'] = $matches[9];
					break;
				} // end switch
			} // end if
		} // end if
		
		// some ftp servers return . and .. as directory entries and also may
		// display a total line. Make sure to send back an empty struct if they are found
		if($rawlist_struct['name'] == '.' || $rawlist_struct['name'] == '..' || substr($rawlist_str,0,5) == 'total'){
			$rawlist_struct = Array("type" => "", "perm" => "", "group" => "", "owner" => "", "size" => "", "time" => "", "name" => "");
		}

		return $rawlist_struct;
	} // end function
	// ---------------------------------------------------------------------------
	// this function takes a string and attempts to fix inconsistencies between
	// end of line (eol) characters  that are used across different plaforms
	function fix_end_of_lines($str){
		$str = str_replace("\r\r","\r\n",$str);
		$str = str_replace("\r\n","\n",$str);
		return $str;
	} // end function
	// ---------------------------------------------------------------------------
	// this function takes a handle to an ftp connection and try's to determine
	// the system type used. If found, either "win" or "unix" are returned. If
	// it couldn't be determined, then "" is returned.
	function get_ftp_system_type($ftp_handle){
		$ftp_system_type = "";
		if($ftp_system_type = @ftp_systype($ftp_handle)){
			$ftp_system_type = strtolower($ftp_system_type);
			if(strpos($ftp_system_type,"win") !== FALSE){ $ftp_system_type = "win"; }
			elseif(strpos($ftp_system_type,"unix") !== FALSE){ $ftp_system_type = "unix"; }
		}
		return $ftp_system_type;
	} // end function
	// ---------------------------------------------------------------------------
	// this function takes a string in the form of a directory path and attempts
	// to find the parent directory by removing the last level beyond the slash.
	// The new parent string is then returned.
	function get_parent_path($curr_path){
		$parent_path = "";

		// remove leading and trailing slashes from the directory
		$curr_path = trim_dir_slashes($curr_path);

		// split the array at each slash 
		$path_pieces = split("/", $curr_path);

		// pop off the last directory level
		array_pop($path_pieces);

		// now fuse the pieces back together
		$parent_path = "/" . join("/",$path_pieces);

		return $parent_path;
	} // end function
	// ---------------------------------------------------------------------------
	// after a list command, the name of a symlink appears in the form of 
	// "path -> real path" like "cpan -> /www/pub/CPAN". This function takes such a
	// string and returns an associative array where the key 'symlink_path'
	// points to the displayed path and 'symlink_real_path' points to the actual path
	function parse_symlink_name($symlink_name){
		$symlink_struct = Array('symlink_path' => '', 'symlink_real_path' => '');

		$symlink_regex = "(.*)[ ]*->[ ]*(.*)";
		if(ereg($symlink_regex, $symlink_name, $matches) == TRUE){
			$symlink_struct['symlink_path'] = $matches[1];
			$symlink_struct['symlink_real_path'] = $matches[2];
		}

		return $symlink_struct;
	} // end function
	// ---------------------------------------------------------------------------
	// this function takes a filename string and attempts to find out what it's
	// file extension is. The extension is returned (without the period and lowercased). If no
	// extension was found, an empty string is returned
	function get_filename_extension($filename){
		$filename = strtolower($filename);
		$file_ext = '';

		if(strlen($filename) > 0){
			// attempt to find the rightmost position of a period in the filename
			$dot_pos = strrpos($filename,'.');
			// don't proceed if no period was found, or it was at the very beginning (most likely a unix system/hidden file)
			if($dot_pos !== FALSe && $dot_pos !== 0){
				$file_ext = substr($filename, $dot_pos+1);
			}
		} // end if
		return $file_ext;
	} // end function
	// ---------------------------------------------------------------------------
	// this function takes an existing ftp_handle and attempts to retrieve the
	// remote_file and store its contents into a temporary file in
	// temp_file_path. The ftp_mode (ascii,binary) specifies how the file is to
	// be retrieved. Upon success, the file contents as a string is returned. If
	// an error has occured, and array will be returned with all of the error messages within
	function get_ftp_file_contents($ftp_handle, $temp_file_path, $remote_file, $ftp_mode) {
		$error_log = Array();
		$file_contents = "";

		// create a temporary file in the temp_file_path directory
		$temp_filename = tempnam($temp_file_path, 'get');
		if($temp_filename !== FALSE){ 

			// select the ftp transfer mode
			$ftp_mode = (strpos(strtolower($ftp_mode),'bin') !== FALSE) ? FTP_BINARY : FTP_ASCII;

			// attempt to GET the file from the ftp server and store it in our temporary file
			if(FALSE !== @ftp_get($ftp_handle, $temp_filename, $remote_file, $ftp_mode)){

				// try to get the contents of the file we've just gotten
				$temp_contents = get_local_file_contents($temp_filename);
				// check for any errors generated and add them to our log
				if(!is_array($temp_contents)){
					$file_contents = $temp_contents;
				} // end if
				else{
					foreach($temp_contents as $idx => $error_msg){ $error_log[] = $error_msg; }
				}
			} // end if
			else{ $error_log[] = "Could not get the remote file '${remote_file}' from the FTP server and save it to the temporary file '$temp_filename'. Make sure the directory '${temp_file_path}' has write permissions enabled."; }
		} // end if
		else{ $error_log[] = "Could not create the temporary file '${temp_filename}'. Make sure the '${temp_file_path}' directory has write permissions enabled."; }

		// as a last step, attempt to remove the temporary file
		@unlink($temp_filename); 

		// if the file was transfered as ASCII, we have to standardize the
		// different end-of-line characters used on different platforms
		if($ftp_mode == FTP_ASCII && strlen($file_contents) > 0){
			$file_contents = fix_end_of_lines($file_contents);
		}

		return (count($error_log) > 0) ? $error_log : $file_contents;
	} // end function 
	// ---------------------------------------------------------------------------
	// this function attempts to read the file specified by $filename, and send
	// the contents of the file back to the user as a string. If errors are
	// encountered, and array of the error messages will be returned instead.
	function get_local_file_contents($filename){
		$error_log = Array();
		$file_contents = "";

		// make sure the file exists
		if(file_exists($filename)){
			$file_handle = fopen($filename, "rb");
			if($file_handle !== FALSE){ 
				// make sure that PHP isn't caching any of the file stats
				clearstatcache();
				// only read the file if we're sure it has some data. Don't
				// error, simply return an empty string.
				$file_size = filesize($filename);
				if($file_size !== FALSE && $file_size > 0){
					// try to read the file's contents
					$temp_contents = fread($file_handle, $file_size);
					if($temp_contents !== FALSE){
						// try to close the file handle
						if(fclose($file_handle) !== FALSE){
							// everything has worked out, so assign the
							// file contents to be returned to the user
							$file_contents = $temp_contents;
						} // end if
						else{ $error_log[] = "Could not close the file handle for '${filename}'."; }
					} // end if
					else{ $error_log[] = "Could not read the contents of the file '${filename}'."; }
				} // end if
			} // end if
			else{ $error_log[] = "Could not open the file '${filename}'. Make sure the directory has read permissions enabled."; }
		} // end if
		else{ $error_log[] = "Could not find the file '${filename}'. Make sure the 'temp_files' directory has write permissions enabled"; }

		return (count($error_log) > 0) ? $error_log : $file_contents;
	} // end function
	// ---------------------------------------------------------------------------
	// this function takes an existing ftp_handle and attempts to write the
	// file_contents passed in to a temporary file located in temp_file_path and then ftp that file to the
	// remote_file. The ftp_mode (ascii,binary) specifies how the file is to
	// be transferred. Upon success, a boolean TRUE is returned. If
	// an error has occured, and array will be returned with all of the error messages within
	function put_ftp_file_contents($ftp_handle, $temp_file_path, $remote_file, $ftp_mode, $file_contents) {
		$error_log = Array();
		$return_data = '';

		// if the file was transfered as ASCII, we have to standardize the
		// different end-of-line characters used on different platforms
		if($ftp_mode == FTP_ASCII && strlen($file_contents) > 0){
			$file_contents = fix_end_of_lines($file_contents);
		}

		// create a temporary file in the temp_file_path directory
		$temp_filename = tempnam($temp_file_path, 'put');
		if($temp_filename !== FALSE){ 

			// try to place our file_contents into the temporary file
			$return_data = put_local_file_contents($temp_filename, $file_contents);
			if(!is_array($return_data)){

				// select the ftp transfer mode
				$ftp_mode = (strpos(strtolower($ftp_mode),'bin') !== FALSE) ? FTP_BINARY : FTP_ASCII;

#echo $remote_file . $temp_filename;
#echo file_get_contents($temp_filename);
	
				// attempt to PUT the file from our temporary file to the ftp server's remote_file
				if(FALSE !== @ftp_put($ftp_handle, $remote_file, $temp_filename, $ftp_mode)){
					// the ftp put has worked, so relax
				}
				else{ $error_log[] = "!! Could not put the temporary file '${temp_filename}'. Make sure the '${temp_file_path}' directory has write permissions enabled."; }

			} // end if
			else{
				// add each error message to our error log
				foreach($return_data as $idx => $error_msg){ $error_log[] = $error_msg; }
			}
		} // end if
		else{ $error_log[] = "Could not create the temporary file '${temp_filename}'. Make sure the '${temp_file_path}' directory has write permissions enabled."; }

		// as a last step, attempt to remove the temporary file
		@unlink($temp_filename); 

		return (count($error_log) > 0) ? $error_log : TRUE;
		#echo $error_log[0];
	} // end function 
	// ---------------------------------------------------------------------------
	// this function attempts to (over)write $file_contents to the file specified by
	// $filename. On success, a boolean TRUE is returned. If errors are
	// encountered, an array of the error messages will be returned instead.
	function put_local_file_contents($filename, $file_contents){
		$error_log = Array();
		
		#echo $filename;
		#echo $file_contents;
		
		// don't do anything if no file_contents have been given
		if(strlen($file_contents) > 0){
			// make sure the file exists
			if(file_exists($filename)){
				$file_handle = fopen($filename, "wb");
				if($file_handle !== FALSE){ 
					// try to write the file's contents to the file
					if(fwrite($file_handle, $file_contents) !== FALSE){
						// try to close the file handle
						if(fclose($file_handle) !== FALSE){
							// everything has worked, so relax
						} // end if
						else{ $error_log[] = "Could not close the file handle for '${filename}'."; }
					} // end if
					else{ $error_log[] = "Could not write to the file '${filename}'. Make sure the 'temp_files' directory has write permissions enabled."; }
				} // end if
				else{ $error_log[] = "Could not open the file '${filename}'. Make sure the 'temp_files' directory has write permissions enabled."; }
			} // end if
			else{ $error_log[] = "Could not find the file '${filename}'. Make sure the 'temp_files' directory has write permissions enabled"; }
		} // end if

		return (count($error_log) > 0) ? $error_log : TRUE;
		#echo $error_log[0];
	} // end function
	// ---------------------------------------------------------------------------
?>
