//============================================================================================
function List(){
	this.length 	= 0;			// {read-only} specifies the number of nodes in the List
	this.head 	= void(0);			// {public} pointer to the List's head node (undefined if List is empty)
	this.cursor	= void(0);			// {public} pointer to the current node (undefined if List is empty)
	this.nodes	= new Object(); 	// {private} an object whose attributes store our List_Nodes
	this.id_pool 	= new Array();	// {private} an array of unused (recycled) node id's.
	this.next_id	= 0;			// {private} counter used to generate new node id's.

	// any arguments passed to our constructor are consecutively inserted into the List
	for(var i=0; i < arguments.length; i++){ this.insert(arguments[i]); }
}
//--------------------------------------------------------------------------------------------
List.prototype.type = 'List';		// {read-only} attribute common to all lists to identify object as a List
//--------------------------------------------------------------------------------------------
List.prototype.insert = function(_val){
	var node, node_id;

	// recycle an old node id or create a new one
	if(this.id_pool.length > 0){ node_id = this.id_pool.pop(); }
	else{ node_id = this.next_id++; }

	node = new List_Node(node_id, _val); // create a new List_Node instance
	this.nodes[node_id] = node; // place the new node as an attribute of our List's 'nodes' object

	if(this.length == 0){ // if the list is empty
		this.cursor = node; // point the head, cursor, next, and prev attributes all to the new node
		this.head = node;
		node.next = node; 
		node.prev = node;
	}
	else{
		node.next = this.cursor.next; // tell our new node who it's next and prev neighbors are
		node.prev = this.cursor;
		this.cursor.next.prev = node; // rewire the new node's neighbors so they're next and prev pointers are correct
		this.cursor.next = node;
		this.cursor = node; // move the cursor to the newly inserted node
	}

	this.length++; // increment our List's length attribute
	return node_id;
}
//--------------------------------------------------------------------------------------------
List.prototype.remove = function(){
	var data = void(0);
	var node_id;

	if(this.length > 0){
		node_id = this.cursor.id; // store the node's unique id
		data = this.cursor.data; // store the node's data element

		if(this.length == 1){ // if we're removing the List's last element
			this.cursor = void(0); // set cursor to undefined
			this.head = void(0);  // set head to undefined
		}
		else{
			if(this.cursor.id == this.head.id){ this.head = this.head.next; } // if we're removing the head, point the head to the next node
			this.cursor.prev.next = this.cursor.next; // rewire the node's neighbors to point "around" the node to be removed
			this.cursor.next.prev = this.cursor.prev;
			this.cursor = this.cursor.next; // point the cursor to the next node
		}

		this.nodes[node_id].next = void(0); // set the removed node's next and prev attributes to undefined
		this.nodes[node_id].prev = void(0);
		delete this.nodes[node_id]; // remove the attribute from List's internal 'nodes' object
		this.id_pool.push(node_id); // return the node's id to the id_pool array for recycling purposes
		this.length--; // decrease the List's length attribute
	}
	return data;
}
//--------------------------------------------------------------------------------------------
List.prototype.clear = function(){ 
	while(this.length > 0){ this.remove(); } // remove all of the List's nodes
	return 1; // clear() always suceeds 
}
//--------------------------------------------------------------------------------------------
List.prototype.push = function(_val){
	if(this.length > 0){ this.cursor = this.head.prev; } // if the list isn't empty, point the cursor at the tail
	return this.insert(_val); // return the node's id generated from the insert into the list
}
//--------------------------------------------------------------------------------------------
List.prototype.pop = function(){
	if(this.length > 0){ this.cursor = this.head.prev; } // if the list isn't empty, point the cursor at the tail
	return this.remove(); // return the node's data (undefined if empty)
}
//--------------------------------------------------------------------------------------------
List.prototype.enqueue = function(_val){
	var node_id = this.push(_val); // take advantage of the circular nature of the list. Push the value to the end of the list and remember it's returned node id.
	this.head = this.head.prev; // then adjust the head to the new value (which was at the tail)
	return node_id;
}
//--------------------------------------------------------------------------------------------
List.prototype.unshift = function(_val){ return this.enqueue(_val); }
//--------------------------------------------------------------------------------------------
List.prototype.dequeue = function(){
	if(this.length > 0){ this.cursor = this.head; } // if the list isn't empty, point the cursor at the head
	return this.remove(); // return the node's data (undefined if empty). Remove automatically adjusts the head pointer
}
//--------------------------------------------------------------------------------------------
List.prototype.shift = function(){ return this.dequeue(); }
//--------------------------------------------------------------------------------------------
List.prototype.replace = function(_val){
	var data = void(0);
	if(this.length > 0){
		data = this.cursor.data; // save the List node's old data
		this.cursor.data = _val; // set the node's new data
	}
	return data; // return the old data (undefined if List is empty)
}
//--------------------------------------------------------------------------------------------
List.prototype.splice = function(_list){
	var node_ids = void(0); // assume failure
	if((_list.type == 'List') && (_list.length > 0)){ // only splice if a nonzero length List is passed in
		node_ids = new Array();
		_list.cursor = _list.head; // point the cursor at head of our List to be spliced
		do{
			node_ids.push(this.insert(_list.cursor.data)); // insert into our List a new node with the data from the splice List. Push the new node's id into our array.
			_list.cursor = _list.cursor.next; // goto the next element in the splice List
		}
		while(_list.cursor.id != _list.head.id); // stop splicing when we get back to the head node
		return_val = 1; // success
	}
	return node_ids;
}
//--------------------------------------------------------------------------------------------
List.prototype.reverse = function(){
	var outcome = (this.length > 0) ? 1 : 0; // consider it a failed attempt if reverse() is called on an empty list 
	var p, q, tmp; // pointers to List_Nodes
	var proceed = 1; // looping flag
	var cursor_id = '';
	if(this.length > 1){
		cursor_id = this.cursor.id; // reversing the list will tamper with the cursor, so remember it's original id
		p = this.head; // create two pointers. One at the head
		q = this.head.prev; // and the other at the tail
		while(proceed){
			// the pointers work from the outside in swapping (reversing) the nodes in the List
			tmp = p.data; // swap the two node's data
			p.data = q.data;
			q.data = tmp;

			tmp = this.nodes[p.id]; // swap what List_Node's are pointed to in the List's 'nodes' object
			this.nodes[p.id] = this.nodes[q.id];
			this.nodes[q.id] = tmp;

			tmp = p.id; // swap the two node's id's
			p.id = q.id;
			q.id = tmp;

			proceed = ((p.id == q.id) || (p.next.id == q.id)) ? 0 : 1; // only proceed if the pointers haven't met in the middle or are direct neighbors

			p = p.next; // adjust the two pointers closer to the middle of the list
			q = q.prev;
		}
		this.cursor = this.nodes[cursor_id]; // restore the cursor position (will appear on the reflected side of the list)
	}
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.gotoId = function(_id){ 
	_id = parseInt(_id); // check for an integer
	var outcome = 0; // assume failure
	if((this.length > 0) && _id && (_id >= 0)){ // if list is not empty and the id is a positive integer
		this.cursor = this.nodes[_id]; // move the cursor
		outcome = 1; // success
	}
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.gotoHead = function(){ 
	var outcome = 0;
	if(this.length > 0){ 
		this.cursor = this.head;
		outcome = 1;
	}
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.gotoTail = function(){ 
	var outcome = 0;
	if(this.length > 0){ 
		this.cursor = this.head.prev;
		outcome = 1;
	}
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.gotoNext = function(){ 
	var outcome = 0;
	if(this.length > 0){ 
		this.cursor = this.cursor.next;
		outcome = 1;
	}
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.gotoPrev = function(){ 
	var outcome = 0;
	if(this.length > 0){ 
		this.cursor = this.cursor.prev;
		outcome = 1;
	}
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.getCursor = function(){ return (this.length == 0) ? void(0) : this.cursor.data; }
//--------------------------------------------------------------------------------------------
List.prototype.getId = function(){ return (this.length == 0) ? void(0) : this.cursor.id; }
//--------------------------------------------------------------------------------------------
List.prototype.getLength = function(){ return this.length; }
//--------------------------------------------------------------------------------------------
List.prototype.atHead = function(){ 
	var outcome = void(0);
	if(this.length > 0){ outcome = (this.cursor.id == this.head.id) ? 1 : 0; }
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.atTail = function(){ 
	var outcome = void(0);
	if(this.length > 0){ outcome = (this.cursor.id == this.head.prev.id) ? 1 : 0; }
	return outcome;
}
//--------------------------------------------------------------------------------------------
List.prototype.isEmpty = function(){ return (this.length == 0) ? 1 : 0; }
//============================================================================================

//============================================================================================
function List_Node(_id, _data){
	this.id = _id; // unique id to distinguish node
	this.data = _data; // arbitrary data that the node is storing
	this.next = void(0); // a pointer to the node's "next" neighbor
	this.prev = void(0); // a pointer to the node's "prev" (previous) neighbor
}
//============================================================================================
